@@ -1,13 +1,14 @@
 /*
- *  $Id: defFind.c,v 4.17 2007/11/13 05:49:46 bkorb Exp $
+ *  $Id: defFind.c,v 4.19 2008/08/27 14:45:19 bkorb Exp $
  *
- *  Time-stamp:        "2007-11-12 20:44:29 bkorb"
- *  Last Committed:    $Date: 2007/11/13 05:49:46 $
+ *  Time-stamp:        "2008-08-27 07:40:28 bkorb"
+ *  Last Committed:    $Date: 2008/08/27 14:45:19 $
  *
  *  This module locates definitions.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -559,7 +560,7 @@ findDefEntry( char* pzName, ag_bool* pIsIndexed )
 
 
 /*
- *  findEntryList
+ *  entryListSearch
  *
  *  Find the definition entry for the name passed in.  It is okay to find
  *  block entries IFF they are found on the current level.  Once you start
@@ -584,6 +585,13 @@ entryListSearch( char* pzName, tDefCtx* pDefCtx )
      *  an index yet).
      */
     if (defList.nestLevel == 0) {
+        if (! IS_VAR_FIRST_CHAR(*pzName)) {
+            strncpy(zDefinitionName, pzName, sizeof(zDefinitionName) - 1);
+            zDefinitionName[ sizeof(zDefinitionName) - 1] = NUL;
+            ILLFORMEDNAME();
+            return NULL;
+        }
+
         canonicalizeName( zDefinitionName, pzName, (int)strlen( pzName ));
         pzName = zDefinitionName;
         defList.usedCt = 0;
