@@ -1,15 +1,16 @@
 /*
- *  $Id: defLoad.c,v 4.13 2007/07/04 20:51:11 bkorb Exp $
+ *  $Id: defLoad.c,v 4.17 2008/01/23 00:36:00 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:17:00 bkorb"
- *  Last Committed:    $Date: 2007/07/04 20:51:11 $
+ *  Time-stamp:        "2007-11-12 20:50:31 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:00 $
  *
  *  This module loads the definitions, calls yyparse to decipher them,
  *  and then makes a fixup pass to point all children definitions to
  *  their parent definition.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,7 +35,7 @@ static void*      pAllocList     = NULL;
     ((ENTRY_ALLOC_CT * sizeof( tDefEntry )) + sizeof(void*))
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static tDefEntry*
 insertDef( tDefEntry* pDef );
 /* = = = END-STATIC-FORWARD = = = */
@@ -255,7 +256,7 @@ findPlace( char* name, tCC* pzIndex )
     if (pzIndex == NULL)
         pE->index = NO_INDEX;
 
-    else if (isdigit( *pzIndex ) || (*pzIndex == '-'))
+    else if (IS_DEC_DIGIT_CHAR(*pzIndex) || (*pzIndex == '-'))
         pE->index = strtol( pzIndex, NULL, 0 );
 
     else {
