@@ -1,7 +1,7 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for agen5
- * Generated Sat May 24 10:45:58 PDT 2008
+ * Generated Sun Nov 16 16:03:59 PST 2008
  */
 #ifndef AGEN5_PROTO_H_GUARD
 #define AGEN5_PROTO_H_GUARD 1
@@ -65,6 +65,9 @@ spanQuote( char* pzQte );
 LOCAL tCC*
 skipScheme( tCC* pzSrc,  tCC* pzEnd );
 
+LOCAL int
+count_nl(char const * pz);
+
 LOCAL tCC*
 skipExpression( tCC* pzSrc, size_t len );
 
