@@ -1,37 +1,34 @@
 
 /*
- *  $Id: tpLoad.c,v 4.13 2006/09/24 02:59:00 bkorb Exp $
+ *  $Id: tpLoad.c,v 4.26 2008/01/23 00:36:02 bkorb Exp $
  *
- * Time-stamp:        "2006-09-23 19:49:27 bkorb"
- * Last Committed:    $Date: 2006/09/24 02:59:00 $
+ * Time-stamp:        "2007-10-30 08:17:58 bkorb"
+ * Last Committed:    $Date: 2008/01/23 00:36:02 $
  *
  *  This module will load a template and return a template structure.
- */
-
-/*
- *  AutoGen copyright 1992-2006 Bruce Korb
  *
- *  AutoGen is free software.
- *  You may redistribute it and/or modify it under the terms of the
- *  GNU General Public License, as published by the Free Software
- *  Foundation; either version 2, or (at your option) any later version.
+ * This file is part of AutoGen.
+ * AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ * AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
- *  AutoGen is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * AutoGen is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with AutoGen.  See the file "COPYING".  If not,
- *  write to:  The Free Software Foundation, Inc.,
- *             51 Franklin Street, Fifth Floor,
- *             Boston, MA  02110-1301, USA.
+ * AutoGen is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 static tTlibMark magicMark = TEMPLATE_MAGIC_MARKER;
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static ag_bool
 canReadFile( tCC* pzFName );
 
@@ -53,7 +50,7 @@ findTemplate( tCC* pzTemplName )
     while (pT != NULL) {
         if (streqvcmp( pzTemplName, pT->pzTplName ) == 0)
             break;
-        pT = (tTemplate*)(pT->pNext);
+        pT = (tTemplate*)(void*)(pT->pNext);
     }
     return pT;
 }
@@ -76,7 +73,7 @@ canReadFile( tCC* pzFName )
  *  list trying to find the base template file name.
  */
 LOCAL tSuccess
-findFile( tCC* pzFName, char* pzFullName, tCC** papSuffixList )
+findFile(tCC* pzFName, char* pzFullName, tCC** papSuffixList, tCC * pzReferrer)
 {
     char*   pzRoot;
     char*   pzSfx;
@@ -89,7 +86,7 @@ findFile( tCC* pzFName, char* pzFullName, tCC** papSuffixList )
      *  We will not mess with embedded ones.
      */
     if (*pzFName == '$') {
-        if (! optionMakePath( pzFullName, MAXPATHLEN, pzFName,
+        if (! optionMakePath( pzFullName, (int)AG_PATH_MAX, pzFName,
                               autogenOptions.pzProgPath ))
             return FAILURE;
 
@@ -103,14 +100,6 @@ findFile( tCC* pzFName, char* pzFullName, tCC** papSuffixList )
          */
     }
 
-    /*
-     *  Check for an immediately accessible file
-     */
-    if (canReadFile( pzFName )) {
-        strlcpy( pzFullName, pzFName, MAXPATHLEN );
-        goto findFileReturn;
-    }
-
     /*
      *  Not a complete file name.  If there is not already
      *  a suffix for the file name, then append ".tpl".
@@ -122,28 +111,26 @@ findFile( tCC* pzFName, char* pzFullName, tCC** papSuffixList )
              : strchr( pzFName, '.' );
 
     /*
-     *  IF the file does not already have a suffix,
-     *  THEN try the suffixes that are okay for this file.
+     *  The referrer is useful only if it includes a directory name.
      */
-    if ((pzSfx == NULL) && (papSuffixList != NULL)) {
-        tCC** papSL = papSuffixList;
-
-        char* pzEnd = pzFullName +
-            snprintf( pzFullName, MAXPATHLEN-MAX_SUFFIX_LEN, "%s.", pzFName );
-
-        do  {
-            strcpy( pzEnd, *(papSL++) ); /* must fit */
-            if (canReadFile( pzFullName )) {
-                goto findFileReturn;
-            }
-        } while (*papSL != NULL);
+    if (pzReferrer != NULL) {
+        char * pz = strrchr(pzReferrer, '/');
+        if (pz == NULL)
+            pzReferrer = NULL;
+        else {
+            AGDUPSTR(pzReferrer, pzReferrer, "pzReferrer");
+            pz = strrchr(pzReferrer, '/');
+            *pz = NUL;
+        }
     }
 
     /*
      *  IF the file name does not contain a directory separator,
      *  then we will use the TEMPL_DIR search list to keep hunting.
      */
-    if (pzRoot == NULL) {
+    {
+        static char const curdir[]  = ".";
+        static char const zDirFmt[] = "%s/%s";
 
         /*
          *  Search each directory in our directory search list
@@ -151,28 +138,41 @@ findFile( tCC* pzFName, char* pzFullName, tCC** papSuffixList )
          */
         int     ct     = STACKCT_OPT(  TEMPL_DIRS );
         tCC**   ppzDir = STACKLST_OPT( TEMPL_DIRS ) + ct - 1;
+        tCC*    pzDir  = curdir;
+
+        if (*pzFName == '/')
+            ct = 0;
 
         do  {
-            tSCC    zDirFmt[] = "%s/%s";
-            tCC*    pzDir  = *(ppzDir--);
-            char*   pzEnd;
+            char * pzEnd;
+            void * coerce;
 
             /*
              *  IF one of our template paths starts with '$', then expand it.
              */
             if (*pzDir == '$') {
-                if (! optionMakePath( pzFullName, MAXPATHLEN, pzDir,
-                                      autogenOptions.pzProgPath ))
-                    pzDir = ".";
-                else
-                    AGDUPSTR( pzDir, pzFullName, "find directory name" );
+                if (! optionMakePath( pzFullName, (int)AG_PATH_MAX, pzDir,
+                                      autogenOptions.pzProgPath )) {
+                    coerce = (void *)pzDir;
+                    strcpy(coerce, curdir);
 
-                ppzDir[1] = pzDir; /* save the computed name for later */
+                } else {
+                    AGDUPSTR( pzDir, pzFullName, "find directory name" );
+                    coerce = (void *)ppzDir[1];
+                    free(coerce);
+                    ppzDir[1] = pzDir; /* save the computed name for later */
+                }
             }
 
-            pzEnd  = pzFullName
-                + snprintf( pzFullName, MAXPATHLEN-MAX_SUFFIX_LEN,
-                            zDirFmt, pzDir, pzFName );
+            if ((*pzFName == '/') || (pzDir == curdir)) {
+                strlcpy(pzFullName, pzFName, AG_PATH_MAX - MAX_SUFFIX_LEN);
+                pzEnd = pzFullName + strlen(pzFullName);
+            }
+            else {
+                pzEnd = pzFullName
+                    + snprintf( pzFullName, AG_PATH_MAX - MAX_SUFFIX_LEN,
+                                zDirFmt, pzDir, pzFName );
+            }
 
             if (canReadFile( pzFullName ))
                 goto findFileReturn;
@@ -192,14 +192,35 @@ findFile( tCC* pzFName, char* pzFullName, tCC** papSuffixList )
 
                 } while (*papSL != NULL);
             }
-        } while (--ct > 0);
+
+            if (*pzFName == '/')
+                break;
+
+            /*
+             *  IF there is a referrer, use it before the rest of the
+             *  TEMPL_DIRS We detect first time through by *both* pzDir value
+             *  and ct value.  "ct" will have this same value next time
+             *  through and occasionally "pzDir" will be set to "curdir", but
+             *  never again when ct is STACKCT_OPT(TEMPL_DIRS)
+             */
+            if (   (pzReferrer != NULL)
+                && (pzDir == curdir)
+                && (ct == STACKCT_OPT(TEMPL_DIRS))) {
+
+                pzDir = pzReferrer;
+                ct++;
+
+            } else {
+                pzDir = *(ppzDir--);
+            }
+        } while (--ct >= 0);
     }
 
     res = FAILURE;
 
  findFileReturn:
-    if (deallocAddr != NULL)
-        AGFREE( deallocAddr );
+    AGFREE(deallocAddr);
+    AGFREE(pzReferrer);
     return res;
 }
 
@@ -299,7 +320,7 @@ loadMacros( tTemplate* pT,
 #if defined( DEBUG_ENABLED )
     if (HAVE_OPT( SHOW_DEFS )) {
         tSCC zSum[] = "loaded %d macros from %s\n"
-            "\tBinary template size:  0x%X\n\n";
+            "\tBinary template size:  0x%zX\n\n";
         fprintf( pfTrace, zSum, pT->macroCt, pzF, pT->descSize );
     }
 #endif
@@ -312,51 +333,45 @@ loadMacros( tTemplate* pT,
  *  list trying to find the base template file name.
  */
 LOCAL tTemplate*
-loadTemplate( tCC* pzFileName )
+loadTemplate(tCC* pzFileName, tCC * pzReferrer)
 {
     static tmap_info_t  mapInfo;
-    tTemplate* pRes;
+    static char     zRealFile[ AG_PATH_MAX ];
+    tSCC*           apzSfx[] = { "tpl", "agl", NULL };
 
+    tTemplate*      pRes;
+    tMacro*         pSaveMac = pCurMacro;
+
+    /*
+     *  Find the template file somewhere
+     */
+    if (! SUCCESSFUL( findFile(pzFileName, zRealFile, apzSfx, pzReferrer)))
+        AG_ABEND( aprf( zCannot, ENOENT, "map data file", pzFileName,
+                        strerror( ENOENT )));
+
+    text_mmap( zRealFile, PROT_READ|PROT_WRITE, MAP_PRIVATE, &mapInfo );
+    if (TEXT_MMAP_FAILED_ADDR(mapInfo.txt_data))
+        AG_ABEND( aprf( "Could not open template '%s'", zRealFile ));
+
+    /*
+     *  Process the leading pseudo-macro.  The template proper
+     *  starts immediately after it.
+     */
+    pCurMacro    = NULL;
+
+    /*
+     *  Count the number of macros in the template.  Compute
+     *  the output data size as a function of the number of macros
+     *  and the size of the template data.  These may get reduced
+     *  by comments.
+     */
     {
-        tSCC*       apzSfx[] = { "tpl", "agl", NULL };
-        static char zRealFile[ MAXPATHLEN ];
+        tCC*   pzData   = loadPseudoMacro( (tCC*)mapInfo.txt_data, zRealFile );
+        size_t macroCt  = countMacros( pzData );
+        size_t alocSize = (sizeof( *pRes ) + (macroCt * sizeof( tMacro ))
+            + mapInfo.txt_size - (pzData - (tCC*)mapInfo.txt_data)
+            + strlen(zRealFile) + 0x10) & ~0x0F;
 
-        /*
-         *  Find the template file somewhere
-         */
-        if (! SUCCESSFUL( findFile( pzFileName, zRealFile, apzSfx )))
-            AG_ABEND( aprf( zCannot, ENOENT, "map data file", pzFileName,
-                            strerror( ENOENT )));
-
-        text_mmap( zRealFile, PROT_READ|PROT_WRITE, MAP_PRIVATE, &mapInfo );
-        if (TEXT_MMAP_FAILED_ADDR(mapInfo.txt_data))
-            AG_ABEND( aprf( "Could not open template '%s'", pzFileName ));
-    }
-
-    {
-        size_t       macroCt;
-        tMacro*      pSaveMac = pCurMacro;
-        size_t       alocSize;
-        tCC*         pzData;
-
-        /*
-         *  Process the leading pseudo-macro.  The template proper
-         *  starts immediately after it.
-         */
-        pCurMacro    = NULL;
-        pzData = loadPseudoMacro( (tCC*)mapInfo.txt_data, pzFileName );
-
-        /*
-         *  Count the number of macros in the template.  Compute
-         *  the output data size as a function of the number of macros
-         *  and the size of the template data.  These may get reduced
-         *  by comments.
-         */
-        macroCt = countMacros( pzData );
-        alocSize = sizeof( *pRes ) + (macroCt * sizeof( tMacro ))
-                   + mapInfo.txt_size - (pzData - (tCC*)mapInfo.txt_data)
-                   + strlen( pzFileName ) + 0x10;
-        alocSize &= ~0x0F;
         pRes = (tTemplate*)AGALOC( alocSize, "main template" );
         memset( (void*)pRes, 0, alocSize );
 
@@ -366,21 +381,22 @@ loadTemplate( tCC* pzFileName )
         pRes->magic     = magicMark;
         pRes->descSize  = alocSize;
         pRes->macroCt   = macroCt;
+
         strcpy( pRes->zStartMac, zStartMac ); /* must fit */
         strcpy( pRes->zEndMac, zEndMac );     /* must fit */
-        loadMacros( pRes, pzFileName, "*template file*", pzData );
-
-        pRes->pzTplName   -= (long)pRes;
-        pRes->pzTemplText -= (long)pRes;
-        pRes = (tTemplate*)AGREALOC( (void*)pRes, pRes->descSize,
-                                     "resize template" );
-        pRes->pzTplName   += (long)pRes;
-        pRes->pzTemplText += (long)pRes;
-
-        text_munmap( &mapInfo );
-        pCurMacro    = pSaveMac;
+        loadMacros( pRes, zRealFile, "*template file*", pzData );
     }
 
+    pRes->pzTplName   -= (long)pRes;
+    pRes->pzTemplText -= (long)pRes;
+    pRes = (tTemplate*)AGREALOC( (void*)pRes, pRes->descSize,
+                                 "resize template" );
+    pRes->pzTplName   += (long)pRes;
+    pRes->pzTemplText += (long)pRes;
+
+    text_munmap( &mapInfo );
+    pCurMacro    = pSaveMac;
+
     return pRes;
 }
 
@@ -415,12 +431,13 @@ unloadTemplate( tTemplate* pT )
 LOCAL void
 cleanup( tTemplate* pTF )
 {
-#if ! defined(DEBUG_ENABLED)
+    optionFree(&autogenOptions);
+
     for (;;) {
         tTemplate* pT = pNamedTplList;
         if (pT == NULL)
             break;
-        pNamedTplList = (tTemplate*)pT->pNext;
+        pNamedTplList = (tTemplate*)(void*)(pT->pNext);
         unloadTemplate( pT );
     }
 
@@ -430,7 +447,6 @@ cleanup( tTemplate* pTF )
     ag_scmStrings_deinit();
 
     manageAllocatedData( NULL );
-#endif
 }
 
 /*
